#define NPY_NO_DEPRECATED_API NPY_1_7_API_VERSION
#define PY_SSIZE_T_CLEAN
#include <Python.h>
#include <numpy/arrayobject.h>

#include "ClusterReader.h"
#include "RawFileReader.h"

#include "arr_desc.h"
#include "cluster_reader.h"
#include "data_types.h"

static PyObject *clusterize(PyObject *Py_UNUSED(self), PyObject *args) {

    // // Create an uninitialized numpy array
    // PyArray_Descr *dtypeIn = cluster_dt();
    // PyArray_Descr *dtypeOut = cluster_analysis_dt();

    PyObject *cl_obj;
    if (!PyArg_ParseTuple(args, "O", &cl_obj))
        return NULL;

    // Create a numpy array from the passed object, if possible numpy will
    // use the underlying buffer, otherwise it will create a copy, for example
    // if data type is different or we pass in a list. The
    // NPY_ARRAY_C_CONTIGUOUS flag ensures that we have contiguous memory.
    // function steals a reference to the data type so no need to deallocate
    PyObject *cl_array = PyArray_FromArray(
        (PyArrayObject *)cl_obj, cluster_dt(), NPY_ARRAY_C_CONTIGUOUS);
    if (cl_array == NULL) {
        PyErr_SetString(PyExc_TypeError,
                        "Could not convert first argument to numpy array.");
        return NULL;
    }

    const int ndim = PyArray_NDIM((PyArrayObject *)cl_array);
    npy_intp *dims = PyArray_SHAPE((PyArrayObject *)cl_array);
    Py_ssize_t size = dims[0];

    Cluster *clusters = (Cluster *)(PyArray_DATA((PyArrayObject *)(cl_array)));
    PyObject *cl_analysis =
        PyArray_SimpleNewFromDescr(ndim, dims, cluster_analysis_dt());
    PyArray_FILLWBYTE((PyArrayObject *)cl_analysis, 0); // zero initialization

    // // Get a pointer to the array memory
    ClusterAnalysis *buf = PyArray_DATA((PyArrayObject *)cl_analysis);

    int nc = analyze_clusters(size, clusters, buf);
    if (nc != size) {
        PyErr_SetString(PyExc_TypeError, "Parsed wrong size array!");
    }
    Py_DECREF(cl_array);
    return cl_analysis;
}

static PyObject *get_cluster_dt(PyObject *Py_UNUSED(self), PyObject *args) {
    if (!PyArg_ParseTuple(args, ""))
        return NULL;
    return (PyObject *)cluster_dt();
}

static PyObject *get_frame_header_dt(PyObject *Py_UNUSED(self),
                                     PyObject *args) {
    if (!PyArg_ParseTuple(args, ""))
        return NULL;
    return (PyObject *)frame_header_dt();
}

// Module docstring, shown as a part of help(creader)
static char module_docstring[] = "C functions to read cluster files";

// Module methods
static PyMethodDef creader_methods[] = {
    {"clusterize", clusterize, METH_VARARGS, "Do some stuff"},
    {"cluster_dt", get_cluster_dt, METH_VARARGS, "Do some stuff"},
    {"frame_header_dt", get_frame_header_dt, METH_VARARGS, "Do some stuff"},
    {NULL, NULL, 0, NULL} /* Sentinel */
};

// Module defenition
static struct PyModuleDef creader_def = {
    PyModuleDef_HEAD_INIT,
    "_creader",
    module_docstring,
    -1,
    creader_methods, // m_methods
    NULL,            // m_slots
    NULL,            // m_traverse
    NULL,            // m_clear
    NULL             // m_free
};

// Initialize module and add classes
PyMODINIT_FUNC PyInit__creader(void) {

    PyObject *m = PyModule_Create(&creader_def);
    if (m == NULL)
        return NULL;
    import_array();

    m = init_ClusterFileReader(m);
    m = init_RawFileReader(m);
    return m;
}
